//
//  IOBSHttpManager.m
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import "IOBSHttpManager.h"
#import "IOBSUploadResponseInfo.h"
#import "IOBSUploadOption.h"
#define HMFileBoundary @"iobs"
#define HMNewLien @"\r\n"
#define HMEncode(str) [str dataUsingEncoding:NSUTF8StringEncoding]
#define WS(weakSelf)  __weak __typeof(&*self)weakSelf = self;

typedef NS_ENUM(NSUInteger, UploadType) {
    UploadTypeNomal,
    UploadTypeFenpian,
};

typedef void(^retryBlock)(IOBSUploadResponseInfo * info);

@interface IOBSHttpManager()

@property(nonatomic,copy)IOBSCompleteBlock completeBlock;
@property(nonatomic,copy)IOBSInternalProgressBlock progressBlock;
@property(nonatomic,strong)NSDictionary *responseDic;
@property (nonatomic, copy)retryBlock retryBlock;
@property UInt32 timeout;
@property (nonatomic, strong) IOBSUrlConvert converter;
@property (nonatomic) IOBSDnsManager *dns;


@property (nonatomic, copy)IOBSUploadCompletionHandler completeHandle;
@property (nonatomic, copy)IOBSUploadResponseInfo *uploadInfo;
@property (nonatomic, copy)NSError *uploadError;
@property (nonatomic, copy)NSString *name;
@property (nonatomic, copy)NSString *bucket;
@property (nonatomic, copy)NSString *hostStr;
@property (nonatomic, copy)NSString *key;
@property (nonatomic, copy)NSString *coverid;
@property (assign, nonatomic)UploadType flag;
@property (nonatomic, copy)NSArray * backupHosts;
@property (nonatomic, strong)id json;


@end

@implementation IOBSHttpManager

- (instancetype)initWithTimeout:(UInt32)timeout
                   urlConverter:(IOBSUrlConvert)converter
                            dns:(IOBSDnsManager *)dns backupHosts:(NSArray *)hosts{
    if (self = [super init]) {

        _timeout = timeout;
        _converter = converter;
        _dns = dns;
        
        _uploadOption = [[IOBSUploadOption alloc] init];
        _uploadInfo = [[IOBSUploadResponseInfo alloc] init];
        _backupHosts = hosts;
    }
    
    return self;
}

#pragma mark 检查入参
- (BOOL)checkAndNotifyKey:(NSString *)key
                    token:(NSString *)token
                     date:(NSString *)date
                    input:(NSObject *)input
                   bucket:(NSString *)bucket
{
    NSString *desc = nil;
    if (input == nil) {
        desc = @"no input data";
    }
    else if (token == nil || [token isEqualToString:@""]) {
        desc = @"no token";
    }
    else if (bucket == nil || [bucket isEqual:@""]) {
        desc = @"no bucket";
    }
    else if (key == nil || [key isEqual:@""]) {
        desc = @"no key";
    }
    else if (date == nil || [key isEqual:@""]) {
        desc = @"no dateString";
    }
    
    if (desc != nil) {
        NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:desc, NSLocalizedDescriptionKey, nil];
        
        _uploadError = [NSError errorWithDomain:@"domain" code:0 userInfo:userInfo];
        _uploadInfo.error = _uploadError;
        _uploadInfo.statusCode = InvalidArgument;
        
        _completeHandle(_uploadInfo,key,nil);
        return YES;
    }
    return NO;
}

#pragma mark - 小文件接口
- (void) putFileWithData:(NSData *)data
                  bucket:(NSString *)bucket
                     key:(NSString *)key
                   token:(NSString *)token
                    date:(NSString *)date
                complete:(IOBSUploadCompletionHandler)completionHandler
                  option:(IOBSUploadOption *)option
{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:date input:data bucket:bucket]) {
        return;
    }
    _hostStr = @"http://10.20.25.214:8080";
    _bucket = bucket;
    _key = key;
    _coverid = @"";
    NSDictionary *params = @{};
    
    NSString *url = [NSString stringWithFormat:@"%@/%@/%@",_hostStr,bucket,key];
    [self uploadmimeType:@"application/octet-stream" fileData:data params:params urlString:url Authorization:token date:date retry:0];
    
}
//add baseUrl
- (void) putFileWithData:(NSData *)data
                  bucket:(NSString *)bucket
                     key:(NSString *)key
                   token:(NSString *)token
                    date:(NSString *)date
                complete:(IOBSUploadCompletionHandler)completionHandler
                  option:(IOBSUploadOption *)option
             withBaseUrl:(NSString *)baseUrl
{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:date input:data bucket:bucket]) {
        return;
    }
    _hostStr = baseUrl;
    _bucket = bucket;
    _key = key;
    _coverid = @"";
    NSDictionary *params = @{};
    NSString *url = [NSString stringWithFormat:@"%@/%@/%@",baseUrl,bucket,key];
    
    //url = [url stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
    [self uploadmimeType:@"application/octet-stream" fileData:data params:params urlString:url Authorization:token date:date retry:0];
    
}

- (void) putFileWithData:(NSData *)data
                  bucket:(NSString *)bucket
                     key:(NSString *)key
                   token:(NSString *)token
                    date:(NSString *)date
                complete:(IOBSUploadCompletionHandler)completionHandler
                  option:(IOBSUploadOption *)option
             withBaseUrl:(NSString *)baseUrl
                 coverid:(NSString *)coverid{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:date input:data bucket:bucket]) {
        return;
    }
    _hostStr = baseUrl;
    _bucket = bucket;
    _key = key;
    _coverid = coverid;
    NSDictionary *params = @{};
    if (!baseUrl) {
        baseUrl = @"http://10.20.25.214:8080";
    }
    NSString *url = [NSString stringWithFormat:@"%@/%@/%@?coverid=%@",baseUrl,bucket,key,coverid?coverid:@""];
    [self uploadmimeType:@"application/octet-stream" fileData:data params:params urlString:url Authorization:token date:date retry:0];
}

- (void) durativeUploadFileWithData:(NSData *)data
                           fileName:(NSString *)fileName
                             bucket:(NSString *)bucket
                                key:(NSString *)key
                              token:(NSString *)token
                              index:(NSInteger)index
                           complete:(IOBSUploadCompletionHandler)completionHandler
                             option:(IOBSUploadOption *)option
                        withBaseUrl:(NSString *)baseUrl{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:@"" input:data bucket:bucket]) {
        return;
    }
    
    NSDictionary *params = @{@"token":token};
    if (!baseUrl) {
        baseUrl = @"http://iobs.pingan.com/";
    }
    NSString *url = [NSString stringWithFormat:@"%@%@%@?key=%@&index=%ld&token=%@",baseUrl,@"durativeUpload/",bucket,key,index,token];
    [self danPianPost:url withData:data fileName:@"" mimeType:@"" withParams:params];
}

- (void)durativeUploadFileWithData:(NSData *)data fileName:(NSString *)fileName bucket:(NSString *)bucket key:(NSString *)key token:(NSString *)token index:(NSInteger)index complete:(IOBSUploadCompletionHandler)completionHandler option:(IOBSUploadOption *)option withBaseUrl:(NSString *)baseUrl coverid:(NSString *)coverid{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:@"" input:data bucket:bucket]) {
        return;
    }
    
    NSDictionary *params = @{@"token":token};
    if (!baseUrl) {
        baseUrl = @"http://iobs.pingan.com/";
    }
    NSString *url = [NSString stringWithFormat:@"%@%@%@?key=%@&index=%ld&token=%@&coverid=%@",baseUrl,@"durativeUpload/",bucket,key,index,token,coverid?coverid:@""];
    [self danPianPost:url withData:data fileName:@"" mimeType:@"" withParams:params];
}



- (void) finishUploadFileWithData:(NSData *)data
                fileName:(NSString *)fileName
                  bucket:(NSString *)bucket
                     key:(NSString *)key
                   token:(NSString *)token
                   index:(NSInteger)index
                  length:(long)length
                complete:(IOBSUploadCompletionHandler)completionHandler
                  option:(IOBSUploadOption *)option
             withBaseUrl:(NSString *)baseUrl{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:@"" input:data bucket:bucket]) {
        return;
    }
    
    NSDictionary *params = @{@"token":token};
    if (!baseUrl) {
        baseUrl = @"http://iobs.pingan.com/";
    }
    NSString *url = [NSString stringWithFormat:@"%@%@%@?key=%@&index=%ld&length=%ld&token=%@",baseUrl,@"finishUpload/",bucket,key,index,length,token];
    [self danPianPost:url withData:data fileName:@"" mimeType:@"" withParams:params];

}

- (void)finishUploadFileWithData:(NSData *)data fileName:(NSString *)fileName bucket:(NSString *)bucket key:(NSString *)key token:(NSString *)token index:(NSInteger)index length:(long)length complete:(IOBSUploadCompletionHandler)completionHandler option:(IOBSUploadOption *)option withBaseUrl:(NSString *)baseUrl coverid:(NSString *)coverid{
    _uploadOption = option;
    _completeHandle = completionHandler;
    _flag=UploadTypeNomal;
    
    if ([self checkAndNotifyKey:key token:token date:@"" input:data bucket:bucket]) {
        return;
    }
    
    NSDictionary *params = @{@"token":token};
    if (!baseUrl) {
        baseUrl = @"http://iobs.pingan.com/";
    }
    NSString *url = [NSString stringWithFormat:@"%@%@%@?key=%@&index=%ld&length=%ld&token=%@&coverid=%@",baseUrl,@"finishUpload/",bucket,key,index,length,token,coverid?coverid:@""];
    [self danPianPost:url withData:data fileName:@"" mimeType:@"" withParams:params];

}



- (void)            danPianPost:(NSString *)url
                 withData:(NSData *)data
                 fileName:(NSString *)fileName
                 mimeType:(NSString *)mimeType
               withParams:(NSDictionary *)params

{
    
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:[[NSURL alloc] initWithString:url]];
    
    [request setHTTPMethod:@"POST"];
    //
    //    [request setValue:@"application/json" forHTTPHeaderField:@"Accept"];
    //    [request setValue:@"chunked" forHTTPHeaderField:@"Transfer-Encoding"];
    [request setValue:@"application/octet-stream" forHTTPHeaderField:@"Content-Type"];
    
    [request setValue:[NSString stringWithFormat:@"%ld",data.length] forHTTPHeaderField:@"Content-Length"];
    //    NSLog(@"%ld",data.length);
    NSURLSessionConfiguration *con = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:con delegate:self delegateQueue:nil];
    NSURLSessionUploadTask *task = [session uploadTaskWithRequest:request fromData:data];
    [task resume];
}



#pragma mark 拼接表单
- (void)uploadmimeType:(NSString *)mimeType fileData:(NSData *)fileData params:(NSDictionary *)params urlString:(NSString *)urlString Authorization:(NSString *)Authorization date:(NSString *)date retry:(NSInteger)retryCount
{
    
    // 1.请求路径
    urlString = [urlString stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];

    NSURL *url = [NSURL URLWithString:urlString];
    
    // 2.创建一个POST请求
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url];
    request.HTTPMethod = @"PUT";
    
    // 3.设置请求体
    NSMutableData *body = [NSMutableData data];
    
    
    [body appendData:fileData];
    
    request.HTTPBody = body;
    WS(weakSelf);
    _retryBlock = ^(IOBSUploadResponseInfo *info){
        if (info.couldRetry && retryCount < kIOBSRetryMax) {
            info.retryCount = retryCount +1;
            NSString * backupHots = weakSelf.hostStr;
            if (weakSelf.backupHosts && weakSelf.backupHosts.count > retryCount) {
                backupHots = weakSelf.backupHosts[retryCount];
            }
            NSString *url = [NSString stringWithFormat:@"%@/%@/%@",backupHots,weakSelf.bucket,weakSelf.key];

            
            [weakSelf uploadmimeType:@"application/octet-stream" fileData:fileData params:params urlString:url Authorization:Authorization date:date retry:retryCount+1];
        }
    };
    // 4.设置请求头(告诉服务器这次传给你的是文件数据，告诉服务器现在发送的是一个文件上传请求)
    NSString *contentType = [NSString stringWithFormat:@"application/octet-stream"];
    [request setValue:contentType forHTTPHeaderField:@"Content-Type"];
    [request setValue:Authorization forHTTPHeaderField:@"Authorization"];
    [request setValue:date forHTTPHeaderField:@"Date"];
    
    NSLog(@"Data上传\nrequest:\t%@\nheader:\t%@\nbody:\t%@\nresponse:\n\nerror:\n",request.URL.absoluteString,request.allHTTPHeaderFields, [[NSString alloc] initWithData:[request HTTPBody] encoding:NSUTF8StringEncoding]);
    
    NSURLSessionConfiguration *con = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:con delegate:self delegateQueue:nil];
    NSURLSessionUploadTask *task = [session uploadTaskWithStreamedRequest:request];
    [task resume];
    
}



#pragma mark - 合成文件接口
- (void)             post:(NSString *)url
                    token:(NSString *)token
                     date:(NSString *)date
               withParams:(NSDictionary *)params
        withCompleteBlock:(IOBSCompleteBlock)completeBlock
{
    _flag = UploadTypeFenpian;

    self.completeBlock = completeBlock;
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:[[NSURL alloc] initWithString:url]];
    NSMutableString *param = [NSMutableString string];
    
    NSArray *keyArr = [params allKeys];
    for (int i = 0; i < keyArr.count; i++) {
        
        NSString *key = keyArr[i];
        NSString *value = params[key];
        NSString *paramStr;
        
        if (i == 0) {
            paramStr = [NSString stringWithFormat:@"%@=%@",key,value];
            [param appendString:paramStr];
        }else{
            paramStr = [NSString stringWithFormat:@"%@%@=%@",@"&",key,value];
            [param appendString:paramStr];
        }
    }
    NSMutableData *body = [NSMutableData data];
    [body appendData:[param dataUsingEncoding:NSUTF8StringEncoding]];
    [request setHTTPBody:body];
    [request setHTTPMethod:@"PUT"];
    [request setValue:@"application/octet-stream" forHTTPHeaderField:@"Content-Type"];
    [request setValue:token forHTTPHeaderField:@"Authorization"];
    [request setValue:date forHTTPHeaderField:@"Date"];
    
    NSLog(@"makeFile===\nrequest:\t%@\nheader:\t%@\nbody:\t%@\nresponse:\n\nerror:\n",request.URL.absoluteString,request.allHTTPHeaderFields, [[NSString alloc] initWithData:[request HTTPBody] encoding:NSUTF8StringEncoding]);
    
    NSURLSessionConfiguration *con = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:con delegate:self delegateQueue:nil];
    NSURLSessionUploadTask *task = [session uploadTaskWithStreamedRequest:request];
    [task resume];
    


}
#pragma mark - 块和片接口
- (void)             post:(NSString *)url
                 withData:(NSData *)data
                    token:(NSString *)token
                     date:(NSString *)date
                 mimeType:(NSString *)mimeType
               withParams:(NSDictionary *)params
        withCompleteBlock:(IOBSCompleteBlock)completeBlock
        withProgressBlock:(IOBSInternalProgressBlock)progressBlock
{
    
    _flag = UploadTypeFenpian;
    self.progressBlock = progressBlock;
    self.completeBlock = completeBlock;
    NSMutableURLRequest *request = [[NSMutableURLRequest alloc] initWithURL:[[NSURL alloc] initWithString:url]];
    
    [request setHTTPMethod:@"PUT"];
    //
//    [request setValue:@"application/json" forHTTPHeaderField:@"Accept"];
//    [request setValue:@"chunked" forHTTPHeaderField:@"Transfer-Encoding"];
    [request setValue:@"application/octet-stream" forHTTPHeaderField:@"Content-Type"];

    [request setValue:token forHTTPHeaderField:@"Authorization"];
    [request setValue:date forHTTPHeaderField:@"Date"];

    [request setValue:[NSString stringWithFormat:@"%ld",data.length] forHTTPHeaderField:@"Content-Length"];
//    NSLog(@"%ld",data.length);
    //NSLog(@"块和片上传\nrequest:\t%@\nheader:\t%@\nbody:\t%@\nresponse:\n\nerror:\n",request.URL.absoluteString,request.allHTTPHeaderFields, [[NSString alloc] initWithData:[request HTTPBody] encoding:NSUTF8StringEncoding]);
    NSURLSessionConfiguration *con = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *session = [NSURLSession sessionWithConfiguration:con delegate:self delegateQueue:nil];
    NSURLSessionUploadTask *task = [session uploadTaskWithRequest:request fromData:data];
    [task resume];
}

#pragma mark - 辅助
-(id)JSONObjectWithData:(NSData*)data
{
    return [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:nil];
}


#pragma mark NSURLSessionDelegate

- (void)URLSession:(NSURLSession *)session
          dataTask:(NSURLSessionDataTask *)dataTask
    didReceiveData:(NSData *)data {
    NSHTTPURLResponse *response = (NSHTTPURLResponse *) dataTask.response;
    NSInteger code = response.statusCode;
    //    NSLog(@"-------%@",self.responseDic);
    
    if (code == 200) {
        _json = [self JSONObjectWithData:data];
        _uploadInfo.statusCode = code;
        
    } else {
        _json = [self JSONObjectWithData:data];
        if (_json) {
            NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:_json[@"error"], NSLocalizedDescriptionKey, nil];
            _uploadError = [NSError errorWithDomain:@"domain" code:code userInfo:userInfo];
            _uploadInfo.error = _uploadError;
        }
        
        _uploadInfo.statusCode = code;
        
    }

    
}
#pragma mark 进度条
- (void)URLSession:(NSURLSession *)session
              task:(NSURLSessionTask *)task
   didSendBodyData:(int64_t)bytesSent
    totalBytesSent:(int64_t)totalBytesSent
totalBytesExpectedToSend:(int64_t)totalBytesExpectedToSend {
    
    if (self.progressBlock) {
        
        self.progressBlock(totalBytesSent, totalBytesExpectedToSend);
    }
    double uploadProgress = totalBytesSent / (double)totalBytesExpectedToSend;
    
    if (_uploadOption.progressHandler && !_uploadOption.cancellationSignal()) {
        dispatch_async(dispatch_get_main_queue(), ^{
            
            self.uploadOption.progressHandler(_name,uploadProgress);
        });
        
    }
    else if (_uploadOption.progressHandler && _uploadOption.cancellationSignal()) {
        _uploadInfo.statusCode = Cancelled;
        [task cancel];
    }

}

#pragma mark 完成
- (void)URLSession:(NSURLSession *)session
              task:(NSURLSessionTask *)task
didCompleteWithError:(NSError *)error
{
    if (error) {
        NSLog(@" error >>> %@",error);
        _uploadInfo.error = error;
        _uploadInfo.statusCode = NetworkError;
    }else{
        NSHTTPURLResponse *response = (NSHTTPURLResponse *) task.response;
        NSInteger code = response.statusCode;
        _uploadInfo.statusCode = code;
        _uploadInfo.error = error;
    }
    
    
    if (_flag == UploadTypeNomal && _retryBlock && _uploadInfo.couldRetry &&_uploadInfo.retryCount < kIOBSRetryMax) {
        _retryBlock(_uploadInfo);
        return;
    }
    
    
    if (_flag == UploadTypeNomal) {
        dispatch_async(dispatch_get_main_queue(), ^{
            self.completeHandle(_uploadInfo,_key,_json);
        });
        
    } else {
        dispatch_async(dispatch_get_main_queue(), ^{
            self.completeBlock(_uploadInfo, _json);
        });
        
    }
    [session finishTasksAndInvalidate];
}

#pragma mark IOBSResponseInfo

-(IOBSUploadResponseInfo*)buildResponseInfo:(NSURLResponse *)response
                            withError:(NSError *)error
                         withResponse:(id)responseObject
{
    IOBSUploadResponseInfo *info;
    NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *)response;
    int status =  (int)[httpResponse statusCode];
    
    if (error) {
        
        info = [IOBSUploadResponseInfo responseInfoWithNetError:error];
        
    }else{
        
        info = [[IOBSUploadResponseInfo alloc]initWithStateCode:status withBody:self.responseDic withError:(NSError *)error];
    }
    return info;
}


@end
