//
//  IOBSResponseInfo.h
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import <Foundation/Foundation.h>

/**
 *    中途取消的状态码
 */
extern const int Cancelled;

/**
 *    网络错误状态码
 */
extern const int NetworkError;

/**
 *    错误参数状态码
 */
extern const int InvalidArgument;

/**
 *    错误token状态码
 */
extern const int InvalidToken;

/**
 *    读取文件错误状态码
 */
extern const int InvalidFile;

@interface IOBSUploadResponseInfo : NSObject

/**
 *    状态码
 */
@property (nonatomic,assign) NSInteger statusCode;

/**
 *    错误信息，出错时请反馈此记录
 */
@property (nonatomic, copy) NSError *error;

/**
 *    是否取消
 */
@property (nonatomic, readonly, getter = isCancelled) BOOL canceled;

/**
 *    成功的请求
 */
@property (nonatomic, readonly, getter = isOK) BOOL ok;

/**
 *    是否需要重试，内部使用
 */
@property (nonatomic, readonly) BOOL couldRetry;
@property (nonatomic, assign)NSInteger retryCount;

/**
 *    工厂函数，内部使用
 *
 *    @return 取消的实例
 */
+ (instancetype)cancel;

/**
 *    工厂函数，内部使用
 *
 *    @param desc 错误参数描述
 *
 *    @return 错误参数实例
 */
+ (instancetype)responseInfoWithInvalidArgument:(NSString *)desc;
+ (instancetype)responseInfoWithNetError:(NSError *)error;
/**
 *    工厂函数，内部使用
 *
 *    @param error 错误信息
 *
 *    @return 文件错误实例
 */
+ (instancetype)responseInfoWithFileError:(NSError *)error;

-(instancetype)initWithStateCode:(int)status withBody:(NSDictionary *)dic withError:(NSError *)error;

@end
