//
//  IOBSResponseInfo.m
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import "IOBSUploadResponseInfo.h"

const int InvalidToken = -1;
const int InvalidFile = -3;
const int InvalidArgument = -2;
const int Cancelled = -4;
const int NetworkError = -5;

static IOBSUploadResponseInfo *cancelledInfo = nil;

static NSString *domain = @"IOBS";

@implementation IOBSUploadResponseInfo

-(instancetype)initWithStateCode:(int)status withBody:(NSDictionary *)dic withError:(NSError *)error
{
    if (self = [super init]) {
        
        _statusCode = status;
        _error = error;
        
    }
    return self;
}

+ (instancetype)cancel {
    return [[IOBSUploadResponseInfo alloc] initWithCancelled];
}

- (instancetype)initWithCancelled {
    return [self initWithStatus:Cancelled errorDescription:@"cancelled by user"];
}

+ (instancetype)responseInfoWithFileError:(NSError *)error {
    return [[IOBSUploadResponseInfo alloc] initWithStatus:InvalidFile error:error];
}
+ (instancetype)responseInfoWithNetError:(NSError *)error
{
return [[IOBSUploadResponseInfo alloc] initWithStatus:NetworkError error:error];
}
#pragma mark 文件参数失败
+ (instancetype)responseInfoWithInvalidArgument:(NSString *)text {
    return [[IOBSUploadResponseInfo alloc] initWithStatus:InvalidArgument errorDescription:text];
}

- (instancetype)initWithStatus:(int)status
              errorDescription:(NSString *)text {
    NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:text, NSLocalizedDescriptionKey, nil];
    
    NSError *error = [NSError errorWithDomain:@"domain" code:status userInfo:userInfo];
    return [self initWithStatus:status error:error];
}

- (instancetype)initWithStatus:(int)status
                         error:(NSError *)error {
    return [self initWithStatus:status error:error host:nil duration:0];
}

- (instancetype)initWithStatus:(int)status
                         error:(NSError *)error
                          host:(NSString *)host
                      duration:(double)duration {
    if (self = [super init]) {
        _statusCode = status;
        _error = error;

    }
    return self;
}

- (NSString *)description {
    return [NSString stringWithFormat:@"<%@:status: %ld,error:%@>", NSStringFromClass([self class]), _statusCode, _error.localizedDescription];
}

- (BOOL)isCanceled {
    return _statusCode == Cancelled;
}

- (BOOL)isOK {
    return _statusCode == 200;
}

- (BOOL)couldRetry {
    return (_statusCode >= 500 && _statusCode < 600 && _statusCode != 579) || _statusCode == NetworkError || _statusCode == 996 || _statusCode == 406;
}

@end
