//
//  IOBSUploadManager.m
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import "IOBSUploadManager.h"
#import "IOBSConfig.h"
#import "IOBSHttpManager.h"
#import "IOBSUploadResponseInfo.h"
#import "IOBSResumeUpload.h"
#import "IOBSAsyncRun.h"
#import "IOBSFile.h"
#import "IOBSConfiguration.h"
#import "IOBSRecorderDelegate.h"

@interface IOBSUploadManager()

@property (nonatomic) id <IOBSHttpDelegate> httpManager;
@property (nonatomic) IOBSConfiguration *config;

@property (nonatomic ,copy) NSArray * hosts;
@property (nonatomic, copy) NSString *coverid;

@property (nonatomic, copy) IOBSTokenGeter tokenGeter;
@end

@implementation IOBSUploadManager

- (instancetype)initWithTokenGeter:(IOBSTokenGeter)tokenGeter {
    return [self initWithConfiguration:nil tokenGeter:tokenGeter];
}

- (instancetype)initWithRecorder:(id <IOBSRecorderDelegate> )recorder tokenGeter:(IOBSTokenGeter)tokenGeter{
    
    IOBSConfiguration *config = [IOBSConfiguration build: ^(IOBSConfigurationBuilder *builder) {
        builder.recorder = recorder;
    }];
    return [self initWithConfiguration:config tokenGeter:tokenGeter];
}

- (instancetype)initWithBackupHosts:(NSArray *)hosts tokenGeter:(IOBSTokenGeter)tokenGeter{
    IOBSConfiguration *config = [IOBSConfiguration build: ^(IOBSConfigurationBuilder *builder) {
        builder.backupHosts = hosts;
    }];
    return [self initWithConfiguration:config tokenGeter:tokenGeter];
}

- (instancetype)initWithConfiguration:(IOBSConfiguration *)config tokenGeter:(IOBSTokenGeter)tokenGeter{
    
    if (self = [super init]) {
        
        if (config == nil) {
            config = [IOBSConfiguration build: ^(IOBSConfigurationBuilder *builder) {
            }];
        }
        _tokenGeter = tokenGeter;
        _hosts = config.backupHosts;
        _httpManager = [[IOBSHttpManager alloc] initWithTimeout:config.timeoutInterval urlConverter:config.converter dns:config.dns backupHosts:config.backupHosts];
    }
    return self;
}

//+ (instancetype)sharedInstanceWithConfiguration:(IOBSConfiguration *)config
//{
//    static IOBSUploadManager *sharedInstance = nil;
//    
//    static dispatch_once_t onceToken;
//    dispatch_once(&onceToken, ^{
//        sharedInstance = [[self alloc] initWithConfiguration:config];
//    });
//    
//    return sharedInstance;
//}

#pragma mark 检查入参
+ (BOOL)checkAndNotifyErrorBucket:(NSString *)bucket
                              key:(NSString *)key
                            token:(NSString *)token
                            input:(NSObject *)input
                         complete:(IOBSUploadCompletionHandler)completionHandler {
    NSString *desc = nil;
    if (completionHandler == nil) {
        @throw [NSException exceptionWithName:NSInvalidArgumentException
                                       reason:@"no completionHandler" userInfo:nil];
        return YES;
    }
    if (input == nil) {
        desc = @"no input data";
    }
    else if (token == nil || [token isEqualToString:@""]) {
        desc = @"no token";
    }else if (key == nil || [key isEqualToString:@""]){
        desc = @"no key";
    }else if (bucket == nil || [bucket isEqualToString:@""]){
        desc = @"no bucket";
    }
    
    if (desc != nil) {
        IOBSAsyncRunInMain( ^{
            completionHandler([IOBSUploadResponseInfo responseInfoWithInvalidArgument:desc], key, nil);
        });
        return YES;
    }
    return NO;
}

- (void)sliceUploadFileWithPath:(NSString *)filePath
         bucket:(NSString *)bucket
            key:(NSString *)key
       complete:(IOBSUploadCompletionHandler)completionHandler
         option:(IOBSUploadOption *)option
{
    _coverid = nil;
    [self fenPianUploadFileWithPath:filePath bucket:bucket key:key complete:completionHandler option:option withBaseUrl:nil];
}
/**
 *    分片上传文件
 *
 *    @param filePath          上传文件的路径
 *    @param bucket            上传到指定bucket名称
 *    @param key               上传对应 bucket 里文件的 key
 *    @param completionHandler 上传完成后的回调函数
 *    @param option            上传时传入的可选参数实现上传文件时候的进度查看
 *    @param baseUrl           基础域名  e.g @"http://iobs.pingan.com"
 */
- (void)sliceUploadFileWithPath:(NSString *)filePath
                         bucket:(NSString *)bucket
                            key:(NSString *)key
                         option:(IOBSUploadOption *)option
                    withBaseUrl:(NSString *)baseUrl
                       complete:(IOBSUploadCompletionHandler)completionHandler{
    
    _coverid = nil;
    [self fenPianUploadFileWithPath:filePath bucket:bucket key:key complete:completionHandler option:option withBaseUrl:baseUrl];
}

- (void)putFileWithPath:(NSString *)filePath
                 bucket:(NSString *)bucket
                    key:(NSString *)key
               complete:(IOBSUploadCompletionHandler)completionHandler
                 option:(IOBSUploadOption *)option
            withBaseUrl:(NSString *)baseUrl
                coverid:(NSString *)coverid{
    _coverid = coverid;
    [self fenPianUploadFileWithPath:filePath bucket:bucket key:key complete:completionHandler option:option withBaseUrl:baseUrl];
}



- (void)fenPianUploadFileWithPath:(NSString *)filePath
                           bucket:(NSString *)bucket
                              key:(NSString *)key
                         complete:(IOBSUploadCompletionHandler)completionHandler
                           option:(IOBSUploadOption *)option
                      withBaseUrl:(NSString *)baseUrl {

    
    NSError *error = nil;
    __block IOBSFile *file = [[IOBSFile alloc]init:filePath error:&error];
    if (error) {
        
        IOBSAsyncRunInMain( ^{
            IOBSUploadResponseInfo *info = [IOBSUploadResponseInfo responseInfoWithFileError:error];
            completionHandler(info, key, nil);
        });
        return;
    }
    
    [self putFileInternal:file
                   bucket:bucket
                 fileName:[file fileName]
                      key:key
                 complete:completionHandler
                   option:option
              withBaseUrl:baseUrl];
}





-(void)putFileWithData:(NSData *)fileData
                bucket:(NSString *)bucket
                   key:(NSString *)key
      complete:(IOBSUploadCompletionHandler)completionHandler
        option:(IOBSUploadOption *)option
{
    NSString * uri = [NSString stringWithFormat:@"/%@/%@",bucket,key];
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    IOBSUploadToken * uploadToken = _tokenGeter(signatureDic);
    [_httpManager putFileWithData:fileData bucket:bucket key:key token:uploadToken.Authorization date:dateStr complete:completionHandler option:option];
    
}

-(void)putFileWithData:(NSData *)fileData
                bucket:(NSString *)bucket
                   key:(NSString *)key
                option:(IOBSUploadOption *)option
           withBaseUrl:(NSString *)baseUrl
              complete:(IOBSUploadCompletionHandler)completionHandler{
    
    NSString * uri = [NSString stringWithFormat:@"/%@/%@",bucket,key];
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };

    IOBSUploadToken * uploadToken = _tokenGeter(signatureDic);

    [_httpManager putFileWithData:fileData bucket:bucket key:key token:uploadToken.Authorization date:dateStr complete:completionHandler option:option withBaseUrl:baseUrl];
}

-(void)putFileWithPath:(NSString *)path
                bucket:(NSString *)bucket
                   key:(NSString *)key
                option:(IOBSUploadOption *)option
           withBaseUrl:(NSString *)baseUrl
              complete:(IOBSUploadCompletionHandler)completionHandler{
    NSData * fileData = [NSData dataWithContentsOfFile:path];
    [self putFileWithData:fileData bucket:bucket key:key option:option withBaseUrl:baseUrl complete:completionHandler];
}






//
- (void)putFileWithData:(NSData *)fileData
                 bucket:(NSString *)bucket
                    key:(NSString *)key
               complete:(IOBSUploadCompletionHandler)completionHandler
                 option:(IOBSUploadOption *)option
            withBaseUrl:(NSString *)baseUrl
                coverid:(NSString *)coverid{
    
    NSString * uri = [NSString stringWithFormat:@"/%@/%@",bucket,key];

    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    IOBSUploadToken * uploadToken = _tokenGeter(signatureDic);
    [_httpManager putFileWithData:fileData bucket:bucket key:key token:uploadToken.Authorization date:dateStr complete:completionHandler option:option withBaseUrl:baseUrl coverid:coverid];
}


- (void)uploadImage:(UIImage *)image
              scale:(CGFloat)scale
             bucket:(NSString *)bucket
                key:(NSString *)key
             option:(IOBSUploadOption *)option
        withBaseUrl:(NSString *)baseUrl
           complete:(IOBSUploadCompletionHandler)completionHandler{
    
    UIImage * compressImage = [self compressImage:image scale:scale];
    NSData * fileData = UIImagePNGRepresentation(compressImage);
    NSString * uri = [NSString stringWithFormat:@"/%@/%@",bucket,key];
    
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    IOBSUploadToken * uploadToken = _tokenGeter(signatureDic);
    
    [_httpManager putFileWithData:fileData bucket:bucket key:key token:uploadToken.Authorization date:dateStr complete:completionHandler option:option withBaseUrl:baseUrl];
}
- (void)uploadImage:(UIImage *)image
              scale:(CGFloat)scale
             bucket:(NSString *)bucket
                key:(NSString *)key
           complete:(IOBSUploadCompletionHandler)completionHandler
             option:(IOBSUploadOption *)option
        withBaseUrl:(NSString *)baseUrl
            coverid:(NSString *)coverid{
    
    UIImage * compressImage = [self compressImage:image scale:scale];
    NSData * fileData = UIImagePNGRepresentation(compressImage);
    NSString * uri = [NSString stringWithFormat:@"/%@/%@",bucket,key];
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    IOBSUploadToken * uploadToken = _tokenGeter(signatureDic);
    [_httpManager putFileWithData:fileData bucket:bucket key:key token:uploadToken.Authorization date:dateStr complete:completionHandler option:option withBaseUrl:baseUrl coverid:coverid];
}

- (void)durativeUploadWithData:(NSData *)fileData
                      fileName:(NSString *)fileName
                        bucket:(NSString *)bucket
                           key:(NSString *)key
                         token:(NSString *)token
                         index:(NSInteger)index
                      complete:(IOBSUploadCompletionHandler)completionHandler
                        option:(IOBSUploadOption *)option
                   withBaseUrl:(NSString *)baseUrl{
     [_httpManager durativeUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index complete:completionHandler option:option withBaseUrl:baseUrl];
}

- (void)durativeUploadWithData:(NSData *)fileData
                      fileName:(NSString *)fileName
                        bucket:(NSString *)bucket
                           key:(NSString *)key
                         token:(NSString *)token
                         index:(NSInteger)index
                      complete:(IOBSUploadCompletionHandler)completionHandler
                        option:(IOBSUploadOption *)option
                   withBaseUrl:(NSString *)baseUrl
                       coverid:(NSString *)coverid{
    [_httpManager durativeUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index complete:completionHandler option:option withBaseUrl:baseUrl coverid:coverid];

}


- (void)durativeUploadWithPath:(NSString *)filePath
                      fileName:(NSString *)fileName
                        bucket:(NSString *)bucket
                           key:(NSString *)key
                         token:(NSString *)token
                         index:(NSInteger)index
                      complete:(IOBSUploadCompletionHandler)completionHandler
                        option:(IOBSUploadOption *)option
                   withBaseUrl:(NSString *)baseUrl{
    NSData * fileData = [NSData dataWithContentsOfFile:filePath];
    [_httpManager durativeUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index complete:completionHandler option:option withBaseUrl:baseUrl];
}

- (void)durativeUploadWithPath:(NSString *)filePath
                      fileName:(NSString *)fileName
                        bucket:(NSString *)bucket
                           key:(NSString *)key
                         token:(NSString *)token
                         index:(NSInteger)index
                      complete:(IOBSUploadCompletionHandler)completionHandler
                        option:(IOBSUploadOption *)option
                   withBaseUrl:(NSString *)baseUrl
                       coverid:(NSString *)coverid{
    NSData * fileData = [NSData dataWithContentsOfFile:filePath];
    [_httpManager durativeUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index complete:completionHandler option:option withBaseUrl:baseUrl coverid:coverid];
}

- (void)finishUploadWithData:(NSData *)fileData
                    fileName:(NSString *)fileName
                      bucket:(NSString *)bucket
                         key:(NSString *)key
                       token:(NSString *)token
                       index:(NSInteger)index
                      length:(long)length
                    complete:(IOBSUploadCompletionHandler)completionHandler
                      option:(IOBSUploadOption *)option
                 withBaseUrl:(NSString *)baseUrl{
    [_httpManager finishUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index length:length complete:completionHandler option:option withBaseUrl:baseUrl];
}

- (void)finishUploadWithData:(NSData *)fileData
                    fileName:(NSString *)fileName
                      bucket:(NSString *)bucket
                         key:(NSString *)key
                       token:(NSString *)token
                       index:(NSInteger)index
                      length:(long)length
                    complete:(IOBSUploadCompletionHandler)completionHandler
                      option:(IOBSUploadOption *)option
                 withBaseUrl:(NSString *)baseUrl
                     coverid:(NSString *)coverid{
    [_httpManager finishUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index length:length complete:completionHandler option:option withBaseUrl:baseUrl coverid:coverid];
}

- (void)finishUploadWithPath:(NSString *)filePath
                    fileName:(NSString *)fileName
                      bucket:(NSString *)bucket
                         key:(NSString *)key
                       token:(NSString *)token
                       index:(NSInteger)index
                      length:(long)length
                    complete:(IOBSUploadCompletionHandler)completionHandler
                      option:(IOBSUploadOption *)option
                 withBaseUrl:(NSString *)baseUrl{
    NSData * fileData = [NSData dataWithContentsOfFile:filePath];
    [_httpManager finishUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index length:length complete:completionHandler option:option withBaseUrl:baseUrl];

}
- (void)finishUploadWithPath:(NSString *)filePath
                    fileName:(NSString *)fileName
                      bucket:(NSString *)bucket
                         key:(NSString *)key
                       token:(NSString *)token
                       index:(NSInteger)index
                      length:(long)length
                    complete:(IOBSUploadCompletionHandler)completionHandler
                      option:(IOBSUploadOption *)option
                 withBaseUrl:(NSString *)baseUrl
                     coverid:(NSString *)coverid{
    NSData * fileData = [NSData dataWithContentsOfFile:filePath];
    [_httpManager finishUploadFileWithData:fileData fileName:fileName bucket:bucket key:key token:token index:index length:length complete:completionHandler option:option withBaseUrl:baseUrl coverid:coverid];
    
}
- (void) putFileInternal:(id<IOBSFileDelegate> )file
                  bucket:(NSString *)bucket
                fileName:(NSString *)name
                     key:(NSString *)key
                complete:(IOBSUploadCompletionHandler)completionHandler
                  option:(IOBSUploadOption *)option
             withBaseUrl:(NSString *)baseUrl {
    
    //上传完成
    IOBSUploadCompletionHandler complete = ^(IOBSUploadResponseInfo *info, NSString *key, NSDictionary *resp)
    {
        [file close];
        IOBSAsyncRunInMain( ^{
            completionHandler(info, key, resp);
        });
    };
    

//    NSLog(@"--------%lu",(unsigned long)[file readAll].length);
    //大文件上传
    IOBSResumeUpload *up = [[IOBSResumeUpload alloc]initWithFile:file
                                                            data:[file readAll]
                                                      withBucket:bucket
                                                    withFileName:name
                                                         withKey:key
                                           withCompletionHandler:complete
                                                      withOption:option
                                                 withHttpManager:_httpManager
                                                     withBaseUrl:(NSString *)baseUrl
                                                      backupHost:_hosts];
    
    up.tokenGeter = _tokenGeter;
    if (_coverid) {
        up.coverid = _coverid;
    }
    
    IOBSAsyncRun( ^{
        [up run];
    });

   
}


- (UIImage *)compressImage:(UIImage *)image scale:(CGFloat)scale{
    CGSize size = CGSizeMake(ceilf(image.size.width * scale), ceilf(image.size.height * scale));
    UIGraphicsBeginImageContext(size);
    [image drawInRect:CGRectMake(0, 0, size.width, size.height)];
    UIImage * resultImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return resultImage;
}


static NSString * AFRFC822FormatStringFromDate() {
    NSDate *date = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
    [dateFormatter setDateFormat:@"EEE, dd MMM yyyy HH:mm:ss z"];
    [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
    
    //return [[dateFormatter stringFromDate:date] stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
    return [dateFormatter stringFromDate:date];
}


@end
