//
//  IOBSConfiguration.h
//  IOBSSDK
//
//  Created by Nicholas on 15/11/2.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "IOBSRecorderDelegate.h"


/**
 *    转换为用户需要的url
 *
 *    @param url  上传url
 *
 *    @return 根据上传url算出代理url
 */
typedef NSString *(^IOBSUrlConvert)(NSString *url);

@class IOBSConfigurationBuilder;
@class IOBSDnsManager;
@class IOBSServiceAddress;

/**
 *    Builder block
 *
 *    @param builder builder实例
 */
typedef void (^IOBSConfigurationBuilderBlock)(IOBSConfigurationBuilder *builder);

@interface IOBSConfiguration : NSObject




/**
 *    备用上host
 */
@property (nonatomic, copy)NSArray * backupHosts;
/**
 *    断点上传时的分片大小
 */
@property (readonly) UInt32 chunkSize;

/**
 *    如果大于此值就使用断点上传，否则使用form上传
 */
@property (readonly) UInt32 putThreshold;

/**
 *    上传失败的重试次数
 */
@property (readonly) UInt32 retryMax;

/**
 *    超时时间 单位 秒
 */
@property (readonly) UInt32 timeoutInterval;

@property (nonatomic, readonly) id <IOBSRecorderDelegate> recorder;

@property (nonatomic, readonly)  NSDictionary *proxy;

@property (nonatomic, readonly) IOBSUrlConvert converter;

@property (nonatomic, readonly) IOBSDnsManager *dns;

+ (instancetype)build:(IOBSConfigurationBuilderBlock)block;

@end

/**
 * 上传服务地址
 */
@interface IOBSServiceAddress : NSObject

- (instancetype) init:(NSString*)address ips:(NSArray*)ips;

@property (nonatomic, readonly) NSString* address;
@property (nonatomic, readonly) NSArray* ips;

@end

@interface IOBSZone : NSObject

/**
 *    默认上传服务器地址
 */
@property (nonatomic, readonly) IOBSServiceAddress *up;

/**
 *    备用上传服务器地址
 */
@property (nonatomic, readonly) IOBSServiceAddress *upBackup;


/**
 *    Zone初始化方法

 *
 *    @return Zone实例
 */
- (instancetype)initWithUp:(IOBSServiceAddress *)up
                  upBackup:(IOBSServiceAddress *)upBackup;

/**
 *    zone 0
 *
 *    @return 实例
 */
+ (instancetype)zone0;

/**
 *    zone 1
 *
 *    @return 实例
 */
+ (instancetype)zone1;

@end

@interface IOBSConfigurationBuilder : NSObject


@property (nonatomic, strong)NSArray * backupHosts;



/**
 *    默认上传服务器地址
 */
@property (nonatomic, strong) IOBSZone *zone;

/**
 *    断点上传时的分片大小
 */
@property (assign) UInt32 chunkSize;

/**
 *    如果大于此值就使用断点上传，否则使用form上传
 */
@property (assign) UInt32 putThreshold;

/**
 *    上传失败的重试次数
 */
@property (assign) UInt32 retryMax;

/**
 *    超时时间 单位 秒
 */
@property (assign) UInt32 timeoutInterval;

@property (nonatomic, assign) id <IOBSRecorderDelegate> recorder;

@property (nonatomic, assign)  NSDictionary *proxy;

@property (nonatomic, assign) IOBSUrlConvert converter;

@property (nonatomic, assign) IOBSDnsManager *dns;

@end
