//
//  IOBSConfiguration.m
//  IOBSSDK
//
//  Created by Nicholas on 15/11/2.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import "IOBSConfiguration.h"

//const UInt32 kQNBlockSize = 4 * 1024 * 1024;

@implementation IOBSConfiguration

+ (instancetype)build:(IOBSConfigurationBuilderBlock)block {
    IOBSConfigurationBuilder *builder = [[IOBSConfigurationBuilder alloc] init];
    block(builder);
    return [[IOBSConfiguration alloc] initWithBuilder:builder];
}



- (instancetype)initWithBuilder:(IOBSConfigurationBuilder *)builder {
    if (self = [super init]) {
        
        _chunkSize = builder.chunkSize;
        _putThreshold = builder.putThreshold;
        _retryMax = builder.retryMax;
        _timeoutInterval = builder.timeoutInterval;
        
        _recorder = builder.recorder;
        
        _proxy = builder.proxy;
        
        _converter = builder.converter;
    
        _backupHosts = builder.backupHosts;
    }
    return self;
}

@end

@implementation IOBSConfigurationBuilder

- (instancetype)init {
    if (self = [super init]) {
        _zone = [IOBSZone zone0];
        _chunkSize = 256 * 1024;
        _putThreshold = 512 * 1024;
        _retryMax = 3;
        _timeoutInterval = 60;
        
        _recorder = nil;
        
        _proxy = nil;
        _converter = nil;
        
    }
    return self;
}

@end

@implementation IOBSServiceAddress : NSObject

- (instancetype) init:(NSString*)address ips:(NSArray*)ips{
    if (self = [super init]) {
        _address = address;
        _ips = ips;
        
    }
    return self;
}

@end

@implementation IOBSZone

- (instancetype)initWithUp:(IOBSServiceAddress *)up
                  upBackup:(IOBSServiceAddress *)upBackup {
    if (self = [super init]) {
        _up = up;
        _upBackup = upBackup;
    }
    
    return self;
}

+ (instancetype)createWithHost:(NSString*)up backupHost:(NSString*)backup ip1:(NSString*)ip1 ip2:(NSString*)ip2{
    NSArray* ips = [NSArray arrayWithObjects:ip1,ip2, nil];
    NSString* a = [NSString stringWithFormat:@"http://%@", up];
    IOBSServiceAddress *s1 = [[IOBSServiceAddress alloc] init:a ips:ips];
    NSString* b = [NSString stringWithFormat:@"http://%@", backup];
    IOBSServiceAddress *s2 = [[IOBSServiceAddress alloc] init:b ips:ips];
    return [[IOBSZone alloc] initWithUp:s1 upBackup:s2];
}

+ (instancetype)zone0 {
    static IOBSZone *z0 = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        z0 = [IOBSZone createWithHost:@"upload.qiniu.com" backupHost:@"up.qiniu.com" ip1:@"183.136.139.10" ip2:@"115.231.182.136"];
    });
    return z0;
}

+ (instancetype)zone1 {
    static IOBSZone *z1 = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        z1 = [IOBSZone createWithHost:@"upload-z1.qiniu.com" backupHost:@"up-z1.qiniu.com" ip1:@"106.38.227.28" ip2:@"106.38.227.27"];
    });
    return z1;
}
@end
