//
//  IOBSResumeUpload.m
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import "IOBSResumeUpload.h"
#import "IOBSUrlSafeBase64.h"
#import "IOBSConfig.h"
#import "IOBSUploadResponseInfo.h"
#import "IOBSUploadOption.h"
#import <CommonCrypto/CommonDigest.h>
#import "IOBSUrlSafeBase64.h"
#define WS(weakSelf)  __weak __typeof(&*self)weakSelf = self;

@interface IOBSResumeUpload()

@property (nonatomic, strong) id <IOBSHttpDelegate> httpManager;
@property UInt32 size;
@property (nonatomic, strong) NSData *data;
@property (nonatomic, copy) NSString *bucket;
@property (nonatomic, copy) NSString *name;
@property (nonatomic, copy) NSString *key;
@property (nonatomic, copy) NSString *token;
@property (nonatomic, copy) NSString *date;
@property (nonatomic, strong) IOBSUploadOption *option;
@property (nonatomic, strong) IOBSUploadCompletionHandler complete;
@property (nonatomic, strong) NSMutableArray *contexts;
@property UInt32 chunkCrc;
@property (nonatomic, strong) id <IOBSFileDelegate> file;
@property (nonatomic, strong) id <IOBSRecorderDelegate> recorder;
@property (strong, nonatomic) NSString *baseUrl;
@property (nonatomic, copy) NSArray * backupHosts;
//块序号
@property(nonatomic,assign)int blockId;
@property(nonatomic,assign)int chunkId;
//成功回调返回的json
@property(nonatomic,strong)id  json;

@end

@implementation IOBSResumeUpload

//add baseUrl
- (instancetype) initWithFile:(id <IOBSFileDelegate> )file
                         data:(NSData *)data
                   withBucket:(NSString *)bucket
                 withFileName:(NSString *)name
                      withKey:(NSString *)key
                    withToken:(NSString *)token
        withCompletionHandler:(IOBSUploadCompletionHandler)completeBlock
                   withOption:(IOBSUploadOption *)option
              withHttpManager:(id <IOBSHttpDelegate> )http
                   backupHost:(NSArray *)hosts

{
    if (self = [super init]) {
        
        _file = file;
        _data = data;
        _size = (UInt32)[file size];
        _bucket = bucket;
        _name = name;
        _key = key;
        _token = token;
        _option = option != nil ? option :[IOBSUploadOption defaultOptions];
        _complete = completeBlock;
        _httpManager = http;
        _baseUrl = @"";
        _backupHosts = hosts;
    }
    
    return self;
}

//add baseUrl
- (instancetype) initWithFile:(id <IOBSFileDelegate> )file
                         data:(NSData *)data
                   withBucket:(NSString *)bucket
                 withFileName:(NSString *)name
                      withKey:(NSString *)key
        withCompletionHandler:(IOBSUploadCompletionHandler)completeBlock
                   withOption:(IOBSUploadOption *)option
              withHttpManager:(id <IOBSHttpDelegate> )http
                  withBaseUrl:(NSString *)baseUrl
                   backupHost:(NSArray *)hosts

{
    if (self = [super init]) {
        
        _file = file;
        _data = data;
        _size = (UInt32)[file size];
        _bucket = bucket;
        _name = name;
        _key = key;
        _option = option != nil ? option :[IOBSUploadOption defaultOptions];
        _complete = completeBlock;
        _httpManager = http;
        if (baseUrl) {
            _baseUrl = baseUrl;
        } else {
            _baseUrl = @"";
        }
        
        _backupHosts = hosts;
    }
    
    return self;
}


- (void)run
{
    UInt32 offset = [self recoveryFromRecord];
    self.json = [self readCtx];
    [self nextTask:offset retriedTimes:0 host:_baseUrl];
//    self.json = @{@"blockId":[NSString stringWithFormat:@"%@",offset/kIOBSBlockSize],
//                  @"isUploadOver":@"1",
//                  @"sliceId":[NSString stringWithFormat:@"%@"],}
//    @"{\"blockId\":\"%d\",\"sliceId\":\"%d\",\"isUploadOver\":\"%@\"}"
   
}

- (void)nextTask:(UInt32)offset retriedTimes:(int)retried host:(NSString *)host{
    
    //取消
    if (self.option.cancellationSignal()) {
        
        self.complete([IOBSUploadResponseInfo cancel], self.key, nil);
        
        return;
    }
    

    //完成
    if (offset == self.size){
        
        IOBSCompleteBlock completionHandler = ^(IOBSUploadResponseInfo *info, NSDictionary *resp) {
            
            if (info.isOK) {
                [self removeRecord];
                if (self.option && self.option.progressHandler) {
                    self.option.progressHandler(self.key, 1.0);
                }

            }else if (info.couldRetry && retried < kIOBSRetryMax) {
                
                [self nextTask:offset retriedTimes:retried + 1 host:host];
                return;
            }else {
                [self removeRecord];
            }
            
            self.complete(info, self.key, resp);
        };
        
        [self makeFile:@"" complete:completionHandler];
        return;
    }
    
    //片大小
    UInt32 chunkSize = [self calcPutSize:offset];
    IOBSInternalProgressBlock progressBlock = nil;
    
    //进度
    if (self.option && self.option.progressHandler) {
        
        progressBlock = ^(long long totalBytesWritten, long long totalBytesExpectedToWrite) {
            float percent = (float)(offset + totalBytesWritten) / (float)self.size;
            dispatch_async(dispatch_get_main_queue(), ^{
                
                self.option.progressHandler(self.key, percent);
            });

            
        }; 
    }
    
    //声明完成片上传后的回调
    
    WS(weakSelf);

    IOBSCompleteBlock completionHandler = ^(IOBSUploadResponseInfo *info, NSDictionary *resp) {
        
//        NSLog(@"---------%@  %@",info,resp);
        self.json = resp ? resp : @{};
        
        if (info.isOK) {
            //保存记录
            self.chunkId++;
            [self record:offset + chunkSize ctx:self.json];
            [self nextTask:offset + chunkSize retriedTimes:0 host:host];
            
        }
        else if (info.couldRetry && retried < kIOBSRetryMax) {
            
            NSLog(@"重试>>>%d",retried);
            NSString * backupHots = host;
            if (weakSelf.backupHosts && weakSelf.backupHosts.count > retried) {
                backupHots = weakSelf.backupHosts[retried];
            }
            
            [self nextTask:offset retriedTimes:retried + 1 host:backupHots];
            return;
        }else {
            self.complete(info, self.key, resp);
            return;
        }

        
    };
    
    //块序号
    if (offset % kIOBSBlockSize == 0) {
        
        //块大小
        int64_t blockSize = [self calcBlockSize:offset];
        
        self.blockId = offset/kIOBSBlockSize;
        self.blockId++;
        self.chunkId = 1;
        
        //上传第一块中的第一片
        
        [self makeBlock:_bucket
                    key:_key
                blockId:self.blockId
                 offset:offset
              blockSize:blockSize
              chunkSize:chunkSize
               progress:progressBlock
               complete:completionHandler hots:host];
        return;
    }
    
    
//    NSLog(@"-----------------");
    
    //上传片
    [self putChunk:_bucket  
               key:_key
           context:self.json
            offset:offset
              size:chunkSize
          progress:progressBlock
          complete:completionHandler hots:host];
}

#pragma mark 保存记录(保存的是当前上传片)
- (void)record:(int64_t)offset ctx:(NSDictionary *)ctx
{
    NSString *recorderKey = _key;
    NSMutableDictionary *info = [NSMutableDictionary dictionary];
    //沙盒中的文件路径
    NSString *filePath = [NSHomeDirectory() stringByAppendingString:@"/Documents/"];
    NSString *plistPath =[filePath stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.plist",recorderKey]];
    [info setObject:[NSString stringWithFormat:@"%lld",offset] forKey:@"offset"];
    [info setObject:[NSString stringWithFormat:@"%u",(unsigned int)self.size]forKey:@"totalSize"];
    [info setObject:ctx forKey:@"ctx"];
    [info writeToFile:plistPath atomically:YES];
    
}

#pragma mark 删除记录
- (void)removeRecord {
    NSString *filePath = [NSHomeDirectory() stringByAppendingString:@"/Documents/"];
    NSString *plistPath =[filePath stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.plist",_key]];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if ([self isPlistFileExists:plistPath]) {
        [fileManager removeItemAtPath:plistPath error:nil];
    }
}

#pragma mark 读取纪录
- (UInt32)recoveryFromRecord
{
    NSString *recorderKey = _key;
    NSDictionary *info = [NSDictionary dictionary];
    //沙盒中的文件路径
    NSString *filePath = [NSHomeDirectory() stringByAppendingString:@"/Documents/"];
    NSString *plistPath =[filePath stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.plist",recorderKey]];
    if ([self isPlistFileExists:plistPath]) {
        info = [NSDictionary dictionaryWithContentsOfFile:plistPath];
    }
    NSString *n_offset = info[@"offset"];
    NSString *n_size = info[@"totalSize"];
    
    UInt32 offset = [n_offset intValue];
    UInt32 size = [n_size intValue];
    
    if (offset > size || size != self.size) {
        return 0;
    }
    
    return offset;
}

- (NSDictionary *)readCtx
{
    NSDictionary *ctx = [NSDictionary dictionary];
    //沙盒中的文件路径
    NSString *filePath = [NSHomeDirectory() stringByAppendingString:@"/Documents/"];
    NSString *plistPath =[filePath stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.plist",_key]];
    if ([self isPlistFileExists:plistPath]) {
        ctx = [NSDictionary dictionaryWithContentsOfFile:plistPath][@"ctx"];
    }
    return ctx;
}
#pragma mark 判断本地化是否存在
-(BOOL) isPlistFileExists:(NSString *)path
{
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if( [fileManager fileExistsAtPath:path]) {
//        NSLog(@"not exists");
        return YES;
    }else{
        return NO;
    }
    
}

#pragma mark 计算片
- (UInt32)calcPutSize:(UInt32)offset
{
    UInt32 left = self.size - offset;
    return left < kIOBSChunkSize ? left:kIOBSChunkSize;
}

#pragma mark 计算块的大小 （当块足够时，就是定义块的大小。当不够时。就是剩下的大小）
- (int64_t)calcBlockSize:(UInt32)offset {
    int64_t left = self.size - offset;
    return left < kIOBSBlockSize ? left : kIOBSBlockSize;
}

#pragma mark 上传块中的第一片
- (void)makeBlock:(NSString *)bucket
              key:(NSString *)key
          blockId:(int)blockId
           offset:(int64_t)offset
        blockSize:(int64_t)blockSize
        chunkSize:(int64_t)chunkSize
         progress:(IOBSInternalProgressBlock)progressBlock
         complete:(IOBSCompleteBlock)complete hots:(NSString *)host{
    
    NSString * uri = [NSString stringWithFormat:@"/mkblk/%@/%@/%@",_bucket,key,[NSNumber numberWithInt:blockId]];
    
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    IOBSUploadToken * fenpiantoken = _tokenGeter(signatureDic);
    _token = fenpiantoken.Authorization;
    _date = dateStr;
    //块序列号
    NSString *blockID = [NSString stringWithFormat:@"%d",blockId];
    self.blockId = [blockID intValue];
//    NSString *keyAndBlockId = [NSString stringWithFormat:@"%@/%@",key,blockID];
    NSString *url = [NSString stringWithFormat:@"%@/%@/%@/%@/%@",host,@"mkblk",_bucket,key,blockID];
    NSDictionary *params = @{@"blockIndex":@(self.blockId)
                             };;
    

    NSData *data = [self.file read:offset size:(unsigned int)chunkSize];

    
        [self post:url
        withParams:params
          withData:data
 withCompleteBlock:complete
 withProgressBlock:progressBlock];

}

#pragma mark 上传块中的片(httpDelegate)
- (void)putChunk:(NSString *)bucket
             key:(NSString *)key
         context:(NSDictionary *)ctx
          offset:(UInt32)offset
            size:(UInt32)size
        progress:(IOBSInternalProgressBlock)progressBlock
        complete:(IOBSCompleteBlock)complete hots:(NSString *)host{
    
    
    UInt32 chunkOffset = (offset - ([ctx[@"blockId"] unsignedIntValue] - 1)*kIOBSBlockSize);
    NSData *data = [self.file read:offset size:size];
    NSString *body = [NSString stringWithFormat: @"{\"blockId\":\"%@\",\"sliceId\":\"%@\",\"isUploadOver\":\"%@\"}",ctx[@"blockId"],ctx[@"sliceId"],ctx[@"isUploadOver"]];
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:ctx options:NSJSONWritingPrettyPrinted error:nil];
    NSString *str = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    body = [IOBSUrlSafeBase64 encodeString:str];

    body = [body stringByReplacingOccurrencesOfString:@"=" withString:@"%3D"];
    NSString *baseUrl = [NSString stringWithFormat:@"%@/%@",host,@"bput"];
    NSString *url = [NSString stringWithFormat:@"%@/%@/%@/%@/%d",baseUrl,_bucket,_key,body,(unsigned int)chunkOffset];
    NSDictionary *params = @{@"ctx":body,
                            @"offset":@(chunkOffset)
                            };
    NSString * uri = [NSString stringWithFormat:@"/bput/%@/%@/%@/%d",_bucket,key,body,chunkOffset];
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    
    IOBSUploadToken * fenpiantoken = _tokenGeter(signatureDic);
    _token = fenpiantoken.Authorization;
    _date = dateStr;
    
            [self post:url
            withParams:params
              withData:data
     withCompleteBlock:complete
     withProgressBlock:progressBlock];
}

#pragma mark 上传完成合并文件
- (void)makeFile:(NSString *)uphost
        complete:(IOBSCompleteBlock)complete {
 
     NSString *md5String = [self md5String];
    NSString *baseUrl = [NSString stringWithFormat:@"%@/%@/%@/%@?md5=%@",self.baseUrl,@"mkfile",_bucket,_key,md5String];
    NSDictionary *params = @{@"fileSize":[NSString stringWithFormat:@"%u",(unsigned int)_size],
                             @"fileName":_name,
                             @"md5":md5String,
                             };
    
    NSString * uri = [NSString stringWithFormat:@"/mkfile/%@/%@?md5=%@",_bucket,_key,md5String];
    
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"PUT"
                                    };
    IOBSUploadToken * fenpiantoken = _tokenGeter(signatureDic);
    _token = fenpiantoken.Authorization;
    _date = dateStr;

    
    [self post:baseUrl withParams:params withCompleteBlock:complete];

}
- (void)             post:(NSString *)url
               withParams:(NSDictionary *)params
        withCompleteBlock:(IOBSCompleteBlock)completeBlock
{

    [_httpManager post:url token:_token date:_date withParams:params withCompleteBlock:completeBlock];
}

#pragma mark 发送post请求
- (void)             post:(NSString *)url
               withParams:(NSDictionary *)params
                 withData:(NSData *)data
        withCompleteBlock:(IOBSCompleteBlock)completeBlock
        withProgressBlock:(IOBSInternalProgressBlock)progressBlock {
    
    [_httpManager post:url
              withData:data
                 token:_token
                  date:_date
              mimeType:@""
            withParams:params
     withCompleteBlock:completeBlock
     withProgressBlock:progressBlock];
}

#pragma mark 计算所有块的md5值
-(NSString *)md5String
{
    int countBlock;
    countBlock = (int) _size/kIOBSBlockSize;
    if (_size%kIOBSBlockSize != 0) {
        countBlock++;
    }
    
    long long size;
    NSMutableString *mutableString = [NSMutableString string];
    for (int i = 1; i <= countBlock ; i++) {
        
        long long off = (i - 1)*kIOBSBlockSize;
        size = (_size - off) < kIOBSBlockSize ? (_size - off):kIOBSBlockSize;
        NSData *data = [self read:off size:size];
        NSString *md5 = [self fileMD5:data];
        [mutableString appendString:md5];
    }
    
    return [self md5:mutableString];
}

- (NSData *)read:(long)offset
            size:(long)size {
   
    return [_data subdataWithRange:NSMakeRange(offset, (unsigned int)size)];
    
}

#pragma mark-- 获取md5值
-(NSString*)fileMD5:(NSData*)data
{
    CC_MD5_CTX md5;
    CC_MD5_Init(&md5);
    CC_MD5_Update(&md5, [data bytes], [data length]);
    unsigned char digest[CC_MD5_DIGEST_LENGTH];
    CC_MD5_Final(digest, &md5);
    NSString* s = [NSString stringWithFormat: @"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
                   digest[0], digest[1],
                   digest[2], digest[3],
                   digest[4], digest[5],
                   digest[6], digest[7],
                   digest[8], digest[9],
                   digest[10], digest[11],
                   digest[12], digest[13],
                   digest[14], digest[15]];
    return s;
}

- (NSString *)md5:(NSString *)str
{
    const char *cStr = [str UTF8String];
    unsigned char result[16];
    CC_MD5(cStr, strlen(cStr), result); // This is the md5 call
    return [NSString stringWithFormat:
            @"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
            result[0], result[1], result[2], result[3],
            result[4], result[5], result[6], result[7],
            result[8], result[9], result[10], result[11],
            result[12], result[13], result[14], result[15]
            ];
}

#pragma mark json字典转字符串
- (NSString*)dictionaryToJson:(NSDictionary *)dic
{
    NSError *parseError = nil;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dic options:NSJSONWritingPrettyPrinted error:&parseError];
    NSString *str = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
    str = [IOBSUrlSafeBase64 encodeString:str];
    
    return str;
}

static NSString * AFRFC822FormatStringFromDate() {
    NSDate *date = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
    [dateFormatter setDateFormat:@"EEE, dd MMM yyyy HH:mm:ss z"];
    [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
    
    //return [[dateFormatter stringFromDate:date] stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
    return [dateFormatter stringFromDate:date];
}

@end
