//
//  IOBSDownloadManager.m
//  lobsSDK
//
//  Created by chenjie on 15/10/20.
//  Copyright © 2015年 PingAn. All rights reserved.
//

#import "IOBSDownloadManager.h"
#import "IOBSDownloadOption.h"
#import "IOBSDownloadResponseInfo.h"

//typedef void(^completionProgress)(double bytes);

@interface IOBSDownloadManager()<NSURLSessionDownloadDelegate>

@property (nonatomic, copy)IOBSDownloadCompletionHandler completionBlock;
@property (nonatomic, copy)IOBSDownloadOption *downloadOption;
@property (nonatomic, copy)IOBSDownloadResponseInfo *downloadInfo;
@property (nonatomic, copy)NSError *downloadError;
@property (nonatomic, copy) IOBSTokenGeter tokenGeter;

@end

@implementation IOBSDownloadManager
{
       NSString     *_savePath;
}

#pragma mark 初始化
- (instancetype)initWithTokenGeter:(IOBSTokenGeter)tokenGeter
{
    if (self = [super init]) {
        _downloadOption = [[IOBSDownloadOption alloc] init];
        _downloadInfo = [[IOBSDownloadResponseInfo alloc] init];
        _tokenGeter = tokenGeter;
    }
    return self;
}

- (id)initWithOption:(IOBSDownloadOption *)option{
    if (self = [super init]) {
        _downloadOption = option;
    }
    return self;
}

#pragma mark 检查路径是否为文件夹
- (BOOL)checkPath:(NSString *)path
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    BOOL isDir = FALSE;
    
    BOOL isDirExist = [fileManager fileExistsAtPath:path isDirectory:&isDir];
    
    if(!(isDirExist && isDir))
        
    {
        NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:@"The savePath is not a folder path", NSLocalizedDescriptionKey, nil];
        
        _downloadError = [NSError errorWithDomain:@"domain" code:0 userInfo:userInfo];
        _downloadInfo.error = _downloadError;
        _completionBlock(_downloadInfo);
        
        return YES;
    } else {
        return NO;
    }
}


#pragma mark 下载文件
- (void)getFileWithUrl:(NSString*)urlString
                 token:(NSString *)token
                  date:(NSString *)date
              savePath:(NSString *)path
     completionHandler:(IOBSDownloadCompletionHandler)completionHandler
        downloadOption:(IOBSDownloadOption *)option
                 
{
    _downloadOption = option;
    _completionBlock = completionHandler;
    _savePath = path;
    

    NSURLSessionConfiguration *sessionConfig = [NSURLSessionConfiguration defaultSessionConfiguration];
    NSURLSession *inProcessSession = [NSURLSession sessionWithConfiguration:sessionConfig delegate:self delegateQueue:nil];
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:urlString]];
    request.HTTPMethod = @"GET";
    [request setValue:token forHTTPHeaderField:@"Authorization"];
    [request setValue:@"application/octet-stream" forHTTPHeaderField:@"Content-Type"];
    [request setValue:date forHTTPHeaderField:@"Date"];
    //NSLog(@"下载。。\nrequest:\t%@\nheader:\t%@\nbody:\t%@\nresponse:\n\nerror:\n",request.URL.absoluteString,request.allHTTPHeaderFields, [[NSString alloc] initWithData:[request HTTPBody] encoding:NSUTF8StringEncoding]);
    NSURLSessionDownloadTask *downloadTask = [inProcessSession downloadTaskWithRequest:request];
    
    [downloadTask resume];
}

- (void)downloadWithBucket:(NSString*)bucket
                       key:(NSString *)key
                      host:(NSString *)host
                  savePath:(NSString *)path
            downloadOption:(IOBSDownloadOption *)option
         completionHandler:(IOBSDownloadCompletionHandler)completionHandler{
    
    NSString * uri = [NSString stringWithFormat:@"/%@/%@",bucket,key];
    NSString * dateStr = AFRFC822FormatStringFromDate();
    NSDictionary * signatureDic = @{@"path":uri,
                                    @"contentType":@"application/octet-stream",
                                    @"date":dateStr,
                                    @"method":@"GET"
                                    };
    IOBSUploadToken * Token = _tokenGeter(signatureDic);

    NSString * url = [NSString stringWithFormat:@"%@/%@/%@",host,bucket,key];
    
    [self getFileWithUrl:url token:Token.Authorization date:dateStr savePath:path completionHandler:completionHandler downloadOption:option];
}




#pragma mark - NSURLSessionDownloadDelegate methods

/* 执行下载任务时有数据写入 */
- (void)URLSession:(NSURLSession *)session
      downloadTask:(NSURLSessionDownloadTask *)downloadTask
      didWriteData:(int64_t)bytesWritten
 totalBytesWritten:(int64_t)totalBytesWritten
totalBytesExpectedToWrite:(int64_t)totalBytesExpectedToWrite
{

    
        // 计算当前下载进度
        double progress = totalBytesWritten / (double)totalBytesExpectedToWrite;

    if (_downloadOption.progressHandler) {
        dispatch_async(dispatch_get_main_queue(), ^{
            
            self.downloadOption.progressHandler(progress);
        });

    }

}

/* 下载完成 */
- (void)URLSession:(NSURLSession *)session
      downloadTask:(NSURLSessionDownloadTask *)downloadTask
didFinishDownloadingToURL:(NSURL *)location
{
    NSHTTPURLResponse *response = (NSHTTPURLResponse *) downloadTask.response;
    NSInteger code = response.statusCode;
    if (code == 200) {
        _downloadInfo.code = code;

        [self moveTempFileAtURL:location toDestination:downloadTask.response.suggestedFilename];
    }else{
        
        id json = [self JSONObjectWithData:[NSData dataWithContentsOfURL:location]];
        NSDictionary *userInfo = [NSDictionary dictionaryWithObjectsAndKeys:json[@"error"], NSLocalizedDescriptionKey, nil];
       
        _downloadError = [NSError errorWithDomain:@"domain" code:code userInfo:userInfo];
        _downloadInfo.code = code;
        _downloadInfo.savePath = nil;
        _downloadInfo.error = _downloadError;
       
//
    }
}

- (void)URLSession:(NSURLSession *)session
              task:(NSURLSessionTask *)task
didCompleteWithError:(NSError *)error
{
    NSHTTPURLResponse *response = (NSHTTPURLResponse *) task.response;
    NSLog(@"%@ error:%@",response,error);
    if (error) {
        _downloadInfo.error = error;
    }
    dispatch_async(dispatch_get_main_queue(), ^{

    self.completionBlock(_downloadInfo);
       });
    [session finishTasksAndInvalidate];
}

#pragma mark 下载成功后移动文件
-(void)moveTempFileAtURL:(NSURL *)location toDestination:(NSString *)fileName
{

    NSFileManager *mgr = [NSFileManager defaultManager];
    

    [mgr moveItemAtPath:location.path toPath:_savePath error:nil];
     _downloadInfo.savePath = _savePath;
//    NSLog(@"-------path %@",path);
}

#pragma mark - 辅助
-(id)JSONObjectWithData:(NSData*)data
{
    return [NSJSONSerialization JSONObjectWithData:data options:NSJSONReadingMutableContainers error:nil];
}
static NSString * AFRFC822FormatStringFromDate() {
    NSDate *date = [NSDate date];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
    [dateFormatter setDateFormat:@"EEE, dd MMM yyyy HH:mm:ss z"];
    [dateFormatter setLocale:[[NSLocale alloc] initWithLocaleIdentifier:@"en_US_POSIX"]];
    
    //return [[dateFormatter stringFromDate:date] stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]];
    return [dateFormatter stringFromDate:date];
}
@end
