//
//  IOBSUploadManager.h
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "IOBSUploadToken.h"
@class IOBSUploadResponseInfo;
@class IOBSUploadOption;
@class IOBSConfiguration;
/**
 *    上传完成后的回调函数
 *
 *    @param info 上下文信息，包括状态码，错误值
 *    @param key  上传时指定的key，原样返回
 *    @param resp 上传成功会返回文件信息，失败为nil; 可以通过此值是否为nil 判断上传结果
 */
typedef void (^IOBSUploadCompletionHandler)(IOBSUploadResponseInfo *info, NSString *key, NSDictionary *resp);

typedef IOBSUploadToken *(^IOBSTokenGeter)(NSDictionary * signature);
/**
 管理上传的类，可以生成一次，持续使用，不必反复创建。
 */
@interface IOBSUploadManager : NSObject

/**
 *    基础初始化方法
 *    @tokenGeter 获取token回调
 *    @return 上传管理类实例
 */
- (instancetype)initWithTokenGeter:(IOBSTokenGeter)tokenGeter;





/**
 *    使用备用域名生成上传实例
 *
 *    @param hosts           备用域名池
 *    @tokenGeter 获取token回调

 *    @return 上传管理类实例
 */
- (instancetype)initWithBackupHosts:(NSArray *)hosts tokenGeter:(IOBSTokenGeter)tokenGeter;







/**
 *    上传data文件
 *
 *    @param fileData          上传文件
 *    @param bucket            上传到指定bucket名称
 *    @param key               上传对应 bucket 里文件的 key

 *    @param completionHandler 上传完成后的回调函数
 *    @param option            上传时传入的可选参数实现上传文件时候的进度查看
 *    @param baseUrl           基础域名 e.g  @"http://iobs.pingan.com"
 */

-(void)putFileWithData:(NSData *)fileData
                bucket:(NSString *)bucket
                   key:(NSString *)key
                option:(IOBSUploadOption *)option
           withBaseUrl:(NSString *)baseUrl
              complete:(IOBSUploadCompletionHandler)completionHandler;



/**
 *    根据路径上传文件
 *
 *    @param path              上传路径
 *    @param bucket            上传到指定bucket名称
 *    @param key               上传对应 bucket 里文件的 key
 
 *    @param completionHandler 上传完成后的回调函数
 *    @param option            上传时传入的可选参数实现上传文件时候的进度查看
 *    @param baseUrl           基础域名 e.g  @"http://iobs.pingan.com"
 */

-(void)putFileWithPath:(NSString *)path
                bucket:(NSString *)bucket
                   key:(NSString *)key
                option:(IOBSUploadOption *)option
           withBaseUrl:(NSString *)baseUrl
              complete:(IOBSUploadCompletionHandler)completionHandler;

/**
 *    分片上传文件
 *
 *    @param filePath          上传文件的路径
 *    @param bucket            上传到指定bucket名称
 *    @param key               上传对应 bucket 里文件的 key
 *    @param completionHandler 上传完成后的回调函数
 *    @param option            上传时传入的可选参数实现上传文件时候的进度查看
 *    @param baseUrl           基础域名  e.g @"http://iobs.pingan.com"
 */
- (void)sliceUploadFileWithPath:(NSString *)filePath
                         bucket:(NSString *)bucket
                            key:(NSString *)key
                         option:(IOBSUploadOption *)option
                    withBaseUrl:(NSString *)baseUrl
                       complete:(IOBSUploadCompletionHandler)completionHandler;
/**
 *    上传图片并压缩
 *
 *    @param image             上传图片
 *    @param scale             压缩比例
 *    @param bucket            上传到指定bucket名称
 *    @param key               上传对应 bucket 里文件的 key
 *    @param completionHandler 上传完成后的回调函数
 *    @param option            上传时传入的可选参数实现上传文件时候的进度查看
 *    @param baseUrl           基础域名 e.g  @"http://iobs.pingan.com"
 */

- (void)uploadImage:(UIImage *)image
              scale:(CGFloat)scale
             bucket:(NSString *)bucket
                key:(NSString *)key
             option:(IOBSUploadOption *)option
        withBaseUrl:(NSString *)baseUrl
           complete:(IOBSUploadCompletionHandler)completionHandler;












@end
