//
//  IOBSUploadOption.h
//  IOBSSDK
//
//  Created by Nicholas on 15/10/31.
//  Copyright © 2015年 Nicholas. All rights reserved.
//

#import <Foundation/Foundation.h>

/**
 *    上传进度回调函数
 *
 *    @param key     上传时指定的存储key
 *    @param percent 进度百分比
 */
typedef void (^IOBSUpProgressHandler)(NSString *key, float percent);

/**
 *    上传中途取消函数
 *
 *    @return 如果想取消，返回True, 否则返回No
 */
typedef BOOL (^IOBSUpCancellationSignal)(void);

/**
 *    可选参数集合，此类初始化后sdk上传使用时 不会对此进行改变；如果参数没有变化以及没有使用依赖，可以重复使用。
 */
@interface IOBSUploadOption : NSObject

/**
 *    进度回调函数
 */
@property (copy, nonatomic) IOBSUpProgressHandler progressHandler;

/**
 *    中途取消函数
 */
@property (copy, nonatomic) IOBSUpCancellationSignal cancellationSignal;

/**
 *    可选参数的初始化方法
 
 *    @param progress     进度函数
 *
 *    @return 可选参数类实例
 */


- (instancetype)initWithProgressHandler:(IOBSUpProgressHandler)progress;

- (instancetype)initWithProgressHandler:(IOBSUpProgressHandler)progress
                     cancellationSignal:(IOBSUpCancellationSignal)cancellation;

/**
 *    内部使用，默认的参数实例
 *
 *    @return 可选参数类实例
 */
+(instancetype)defaultOptions;

@end
